# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

"""

Main User Interface module

This module contains a set of classes with text strings to be shown in dialogs
and notifications windows.

All translatable strings should be stored here, and using gettext:
_("escaping").

"""

from gettext import gettext as _
from codeina.ui.base import *

class NoCodec(Dialog):
    
    """ Display a dialog when an application is missing a codec to decode a file. """

    title = _("Missing codecs")
    msg = _("You do not have the proper codecs needed to play this file.")

class NoInternet(Dialog):

    """ Display a notification when no internet connection has been found """ 

    title = _("No connection detected!")
    msg = _("Codeina might not be able to install the latest plugins.")

class NoProviders(Dialog):

    """ Display a dialog when no provider files have been found """ 

    title = _("Missing providers")
    msg = _("It seems like you do not have any provider files!")

class NoProducts(Dialog):

    """ Display a dialog when no matching products have been found """ 

    title = _("Codeina")
    msg = _("No products found to install")

class PackagesInstalled (Dialog):

    """Display a dialog for the installed packages"""

    title = _("Codeina")
    msg = _("Packages installed :\n")

    def __init__ (self, pkgs):
        self.msg += '\n'.join([ pkg.name for pkg in pkgs])

class ShowMatches(Dialog):

    """ Display a list of available providers
        
    @providers_list = list of tuples ("provider_name", return_code)
    @formatted_text = text to display

    """

    title = _("Codeina plugin installer")
    window_title = _("Codeina plugin installer")
    reason = _("""
    A component is required but not installed on your system.

    Below is a list of providers from which you can obtain this component.

    Choose one of these providers to install the required component.""")
    def __init__ (self, pattern):
        from codeina.providers import CodeinaProvider

        self.pattern = pattern
        self.providers_rank, self.matching_products = \
               CodeinaProvider().get_products_for_pattern(pattern)

    def install(self):
        if not self.matching_products:
            # No products are found for this request
            d = NoProducts()
            d()
            return 0
        for (provider_name, rank) in self.providers_rank:
            if provider_name.lower() == 'fluendo':
                # going to the fluendo shop
                from urllib import quote
                from codeina.configure import CODEINA_SHOP_URL
                from codeina.myutils import build_matches_url
                url = build_matches_url(self.pattern) 
                print "Visit", url
                return 0
            else:
                return 0

    def get_provider_for_response (self, response):
        """
            return the name of the provider for the given response
            None if the response is not known
        """
        for counter in range (len(self.providers_rank)):
            if self.providers_rank[counter][1] == response:
                return self.providers_rank[counter][0]
        return None


class ShowUpdates(Notification):

    """ Display a message when no provider files have been found """ 

    title = _("New Products available!")
    msg = _("Codeina found some updates for:")

    def create_msg (self):
        from codeina import providers
        str = '\n'.join([self.msg] + [ ' - ' + package.name for package in providers.CodeinaProvider ().get_products_with_updates ('fluendo')])
        return str

class FluendoShop(Dialog):

    """ Display a message when no provider files have been found """ 

    title = _("Codeina Web Shop")
    window_title = title
    msg = _("Visit codeina webshop.")
