# vi:si:et:sw=4:sts=4:ts=4

# first of all, select reactor
from twisted.internet import gtk2reactor
try:
    # We put it in so that it doesn't try to install it
    # multiple times.
    gtk2reactor.install ()
except:
    pass
from twisted.internet import reactor

import pygtk; pygtk.require('2.0')
import gtk

from codeina.extern.log import log

from dialog import *
from notification import CodeinaNotification as Notification
from notification import CodeinaUpgradeNotification as UpgradeNotification
from fluendo_codec_buyer import FluendoGTK as FluendoShop
from codeina import ui
from gettext import gettext as _

from codeina import httpdownload
from codeina.ui.igtk.downloadprogress import DownloadProgress
httpdownload.HttpDownload.download_object = DownloadProgress()

# Width and hight of the provider's logos
IMG_SIZE = 80

class NoCodec (CodeinaSimpleDialog, ui.NoCodec):
    pass

class NoProviders (CodeinaWarningDialog, ui.NoProviders):
    pass

class NoProducts(CodeinaWarningDialog, ui.NoProducts):
    pass

class NoInternet(Notification, ui.NoInternet):
    pass

class PackagesInstalled (CodeinaSimpleDialog, ui.PackagesInstalled):

    def __init__(self, pkgs):
        ui.PackagesInstalled.__init__ (self, pkgs)
        CodeinaSimpleDialog.__init__ (self)

class ShowUpdates(UpgradeNotification, ui.ShowUpdates):
    pass

    def create_msg (self):
        from codeina import providers 
        return '\n'.join([self.msg] + [' - ' + product.name for product in \
            providers.CodeinaProvider ().get_products_with_updates('fluendo')])

class ShowMatches(gtk.Dialog, ui.ShowMatches):

    def __init__ (self, pattern):
        def UpperFirstLetter (str):
            return str.upper ()[0] + str[1:]

        from codeina.providers import CodeinaProvider

        self.pattern = pattern
        self.providers = CodeinaProvider().get_providers_for_pattern (pattern)

        reason = self.reason
        #reason = providers.get_reason(required_plugins)
        gstr, vers, appl, name, caps = pattern.split('|')
        self.msg = "<b>"
        self.msg += "The application \"%s\" requires the following plugin:" % appl
        self.msg += "</b>\n"
        self.msg += "  - " + name
        gtk.Dialog.__init__ (self, None, None, 0, None)
        self.set_title (self.window_title)
        # Set the icon of this dialog
        from codeina import configure
        self.set_icon_from_file (configure.make_icon_path('codeina.png'))

        text_buffer = gtk.TextBuffer ()
        text_buffer.set_text (reason)

        stock = gtk.image_new_from_stock (gtk.STOCK_DIALOG_INFO,
                                          gtk.ICON_SIZE_DIALOG)
        lbl = gtk.Label ()
        lbl.set_text ("<span size='large'>"+self.msg+"</span>")
        lbl.set_use_markup (True)

        # Add scrollwindow for textview
        scroll = gtk.ScrolledWindow ()
        scroll.set_size_request (640, 300)
        scroll.set_policy ( gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        text = gtk.TextView ()
        text.set_editable (False)
        text.set_buffer (text_buffer)
        scroll.add (text)

        hbox = gtk.HBox ()
        hbox.set_border_width (8)
        hbox.set_spacing (8)
        hbox.pack_start (stock, False, False)
        hbox.pack_start (lbl, False, False)

        self.vbox.pack_start (hbox)
        self.vbox.pack_start (scroll)

        v = gtk.VBox()
        first_button = None
        self.selected_provider = -1

        def callback(widget, data=-1):
            self.selected_provider = data

        for provider in self.providers:
            if first_button == None:
                provider_radio = gtk.RadioButton(label = None, group = None)
            else:
                provider_radio = gtk.RadioButton(label = None, group = first_button)
            provider_radio.connect("toggled", callback, provider)

            if first_button == None:
                provider_radio.set_active(True)
                self.selected_provider = provider
                first_button = provider_radio

            radioHBox = gtk.HBox ()
            img = gtk.Image()
            img.set_from_pixbuf (provider.get_logo_pixbuf (IMG_SIZE))
            radioHBox.pack_start (img, False)
            label = gtk.Label ()
            label.set_markup (_("Install package from <b>%s</b>\n\tProvides:\n\t\t<i>%s</i>") % (UpperFirstLetter (provider.id), '\n\t\t'.join([p.name for p in provider.products])))
            label.set_selectable (False)
            label.set_padding (10, 0)
            radioHBox.pack_start (label)
            provider_radio.add (radioHBox)
            v.pack_start(provider_radio, False)

        v.show()
        self.vbox.pack_start(v)
        
        #for provider in self.providers_rank:
        self.add_button (_("Install"), gtk.RESPONSE_YES)
        self.add_button (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)

    def install (self):
        # check if there are some products for this request
        if not self.providers:
            NoProducts()()
            return 0

        # products found show selection
        self.vbox.show_all ()
        response = self.run ()
        if response == gtk.RESPONSE_CANCEL:
            return 0
        self.hide ()

        # go to the fluendo shop if we choose this provider
        if self.selected_provider.id == 'fluendo':
            import codeina.ui.igtk as ui
            from codeina.myutils import build_matches_url
            from codeina import web

            url = build_matches_url(self.pattern)
            if web.website_exists (url):
                shop = ui.FluendoShop(codeina_url = url)
                shop()
            else:
                # FIXME warning about site that does not exists
                from dialog import CodeinaWarningDialog
                CodeinaWarningDialog (msg=_('Failed to connect to Fluendo shop')) ()
                return 1
            return 0

        # import the correct installer factory
        if self.selected_provider.id in ['ubuntu', 'debian']:
            import codeina.installer.synaptic_installer as installer_factory
            log.info ('codeina', 'using the synaptic installer')
        elif self.selected_provider.id in ['mandrivalinux', 'mandriva']:
            import codeina.installer.urpmi_installer as installer_factory
            log.info ('codeina', 'using the urpmi installer')
        else:
            log.warning ('codeina', 'can not find the correct action for this provider (%s)' % self.selected_provider.id)
            return 0
        installer = installer_factory.Installer ()

        # install the packages
        if 'install_list' in dir(installer):
            installer.install_list (self.selected_provider.products)
        else:
            for pkg in self.selected_provider.products:
                installer.install (pkg)

        # installation done
        from codeina.ui.igtk import PackagesInstalled
        PackagesInstalled (self.selected_provider.products) ()
        log.debug ('codeina', 'all products installed')
        return 0
