# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

from codeina import ui 
import gtk
import gtkmozembed
import os
from codeina.configure import USR_CODEINA
gtkmozembed.set_profile_path(USR_CODEINA, "mozembed")

from codeina.httpdownload import HttpDownload
from codeina.configure import USR_DOWNLOAD, CODEINA_SHOP_URL
from codeina.extern.log import log

import sha
import re

from exceptions import NotImplementedError
import codeina.installer.user_world
import codeina.installer.apt_installer
import codeina.installer.urpmi_installer
from dialog import *
from window import CodeinaWindow
from gettext import gettext as _

# This is the regular expression we will be looking for
# in each links, that tell us how and where to download the
# tarballs, for installation purposes.
_re_download_url = re.compile ('url=(?P<url>.*?)\|sha1sum=(?P<sha1sum>.*?)\|postvars=(?P<postvars>.*?)\|')
_re_post = re.compile ('(?P<var_name>.*?)=(?P<value>.*?)&?')

def _string_to_dict (str):
    """ Convert a string formatted like the following to a dictionary :
    somevar=toto&someothervar=toto2&
    @param str: The string
    @type str: string
    """
    dict = {}
    for match in _re_post.finditer (str):
        name = match.group ("name")
        value = match.group ("value")
        dict[name] = value
    return dict


class FluendoInstallProgress (CodeinaWindow):

    def __init__ (self, title):
        CodeinaWindow.__init__ (self, title)
        
        self.progress_bar = gtk.ProgressBar ()
        self.progress_bar.set_ellipsize (True)

        self.set_size_request (480, 40)

        self.vbox.set_spacing (7)
        self.vbox.pack_start (self.progress_bar)
        self.progress_bar.show ()
        self.vbox.show_all ()

    def set_advancement (self, fraction, text):
        self.progress_bar.set_text (text)
        self.progress_bar.set_fraction (fraction)

    def __close (self, widget):
        pass

class FluendoGTK (CodeinaWindow, ui.FluendoShop):
    """
    Fluendo Codec Buyer. Loads the website, and sniffs all the URL clicked
    to recognize a particular "Install" link, to download and install archives.
    """

    def __install_package (self, path_package):
        """ Determine what type of package we are dealing with, and install it with
        the correct method.

        FIXME: It would be better to guess the format according to magic numbers
        instead of file extensions as it is done now.
        """
        log.info ('codeina', 'trying to install package from path %s' % path_package)
        if path_package.endswith (".tar.bz2") or path_package.endswith (".tar.gz"):
            i = codeina.installer.user_world.Installer ()
            errors = i.install (path_package)
        elif path_package.endswith (".rpm"):
            i = codeina.installer.urpmi_installer.Installer ()
            errors = i.install (path_package)
        elif path_package.endswith (".deb"):
            i = codeina.installer.apt_installer.Installer ()
            errors = i.install (path_package)
        else:
            raise "Unknown package format"

    def __openuri_callback (self, moz, uri):
        """ Callback called whenever a link is clicked in the mozembed window.

        The trick performed here, is that we are actually waiting for a specific
        URL, formatted in such a way that we will know we have to download packages
        and install them. It also checks if they were downloaded correctly, and
        actually installs them.

        If the URL is just a regular URL, then we just exit this function directly
        so that the mozembed window just loads a new page.

        @param moz: The mozembed window.
        @type moz: GtkMozEmbed
        @param uri: The url that was clicked on.
        @type uri: string
        """
        if not _re_download_url.search (uri):
            # This was not the Install button
            return False # Load the page normally

        downloader = HttpDownload ()

        # Disable the main window.
        moz.set_sensitive (False)

        target_directory = USR_DOWNLOAD
        successful_packages = {}
        failed_packages = {}

        errors = ""
        successes = ""
        
        for match in _re_download_url.finditer (uri):
            # We extract the information about the package.
            url = match.group ("url")
            localfile = os.path.basename (url)
            sha1sum = match.group ("sha1sum")
            postvars = _string_to_dict (match.group ("postvars"))
            filename = os.path.basename (url)
            # check if this file is already in our installed 
            # products
            from codeina.productfile import productFile
            if not productFile ().product_quick_add ( filename):
                result = CodeinaMessageDialog ( _("<i>%s</i> is already installed.\nRe-install this product ?") % 
                            localfile, gtk.BUTTONS_YES_NO, gtk.MESSAGE_INFO) ()
                if not result == gtk.RESPONSE_YES:
                    log.debug ('codeina', 'we do not want to reinstall this product')
                    continue
                else:
                    log.debug ('codeina', 'reinstalling this product')

            try:
                downloader.run (url, target_directory, localfile, sha1sum)

                f = file (os.path.join (target_directory, filename), 'rb')
                digest = sha.new (f.read ()).hexdigest ()
                if digest == sha1sum:
                    successful_packages[filename] = digest
                else:
                    failed_packages[filename] = (digest, sha1sum)
                
            except Exception, e:
                failed_packages[filename] = None
        
        if failed_packages != {}:
            for k, v in failed_packages.iteritems ():
                errors += _("Failed to download package %s\n") % (k)
        progress = None
        if successful_packages != {}:
            progress = FluendoInstallProgress (_("Installation Progress"))
            progress.show_all ()
            total = len (successful_packages)
            i = 1.
            for pkg in successful_packages.keys ():
                progress.set_advancement (i / total, _("Installing %s") % (pkg))
                i += 1.
                self.__install_package (os.path.join (target_directory, pkg))
                try:
                    pass
                    successes += _("Package %s installed successfully\n") % (pkg)
                except Exception, e:
                    errors += _("Failed to install package %s !\n") % (pkg)

        if errors != "":
            CodeinaWarningDialog (_("Install and download complete.\n\n%s\n%s") % (errors, successes)) ()
            if progress:
                progress.destroy ()
        else:
            if not successes:
                CodeinaSimpleDialog (_("No new products installed")) ()
            else:
                CodeinaSimpleDialog (_("Install and download complete.\n\n %s") % (successes)) ()
            if progress:
                progress.destroy ()
            gtk.main_quit ()
        
        # FIXME: Change URL as well
        moz.set_sensitive (True) # We can now reenable the window.

        return True # We are not loading anyone !

    def __link_message (self, moz):
        #print moz.get_link_message ()
        pass

    def __init__ (self, codeina_url = CODEINA_SHOP_URL):
        CodeinaWindow.__init__ (self, self.window_title)

        self.connect ("destroy", lambda widget: gtk.main_quit ())
       
        # Mozembed plugins
        self.browser = gtkmozembed.MozEmbed ()
        self.browser.set_size_request (800, 600)
        self.browser.load_url (codeina_url)
        self.browser.connect ('open-uri', self.__openuri_callback)
        self.browser.connect ('link-message', self.__link_message)
        self.browser.show ()

        # Vertical Box
        self.vbox.pack_start (self.browser)
        self.vbox.show ()

    def __call__(self):
        self.show()
        return gtk.main()
